// L09: coffee_beans counter.
#include "opencv\cv.hpp"
#include <iostream>

using namespace cv;
using namespace std;

int main(int argc, char* argv[])
{
	// Objects
	Mat original_img;
	// Load image from disk
	original_img = imread("cafe.tif", CV_LOAD_IMAGE_GRAYSCALE);
	if (!original_img.data){
		cout << "error loading image" << endl;
		system("pause");
		return 1;
	}
	
	// Create image for thresholded image
	Mat inverse_binary(original_img.size(), CV_8UC1);
	// Convert to inverse binary
	int threshold_value = 100;
	threshold(original_img, inverse_binary, threshold_value, 255, THRESH_BINARY_INV);

	// It is necessary to erode several times the image in order to separate each grain
	Mat eroded_image;
	int iterations = 3;
	Mat kernel = getStructuringElement(CV_SHAPE_RECT, Size(5, 5));
	erode(inverse_binary, eroded_image, kernel, Point(-1, -1), iterations);

	// Find all closed shapes in the binary image. The image change at the end of the algorithm.
	vector<vector<Point> > contours;
	findContours(eroded_image.clone(), contours, RETR_EXTERNAL, CHAIN_APPROX_SIMPLE);
	cout << "Number of countours " << contours.size() << endl;
	// Draw contours
	Mat img_contours(inverse_binary.size(), CV_8UC3, Scalar(0, 0, 0));
	for (size_t k = 0; k < contours.size(); k++){
		Scalar color(rand() & 255, rand() & 255, rand() & 255);
		drawContours(img_contours, contours, k, color);
	}

	// Label all the objects
	Mat labels, stats, centroids;
	int connectivity = 8;
	int coffee_beans = connectedComponentsWithStats(eroded_image, labels, stats, centroids, connectivity, CV_32S);
	cout << "number of grains: " << coffee_beans << endl;
	// Draw the centroids
	Mat img_centroids(inverse_binary.size(), CV_8UC3, Scalar(0, 0, 0));
	for (int k = 0; k < centroids.rows; k++){
		Scalar color(rand() & 255, rand() & 255, rand() & 255);
		Point centroid((int)centroids.at<double>(k, 0), (int)centroids.at<double>(k, 1));
		circle(img_centroids, centroid, 3, color, -1, 8);
	}
	// Show images
	imshow("original", original_img);
	imshow("binary_inverse", inverse_binary);
	imshow("erosion", eroded_image);
	imshow("contours", img_contours);
	imshow("crentroids", img_centroids);
	waitKey(0);
	// Free memory
	destroyAllWindows();
	original_img.release();
	inverse_binary.release();
	eroded_image.release();
	img_centroids.release();
	img_contours.release();
	// End of the program
	return 0;
}

